package org.gcube.portlets.user.workspace.client.view;

import org.gcube.portlets.user.workspace.client.AppControllerExplorer;
import org.gcube.portlets.user.workspace.client.ConstantsExplorer;
import org.gcube.portlets.user.workspace.client.ConstantsExplorer.ViewSwitchType;
import org.gcube.portlets.user.workspace.client.event.SwitchViewEvent;
import org.gcube.portlets.user.workspace.client.resources.Resources;
import org.gcube.portlets.user.workspace.client.view.message.MessagesPanel;
import org.gcube.portlets.user.workspace.client.view.smartfolder.SmartFolderPanel;
import org.gcube.portlets.user.workspace.client.view.tree.AsyncTreePanel;
import org.gcube.portlets.user.workspace.client.view.windows.BulkCreatorWindow;

import com.extjs.gxt.ui.client.event.BaseEvent;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.Radio;
import com.extjs.gxt.ui.client.widget.form.RadioGroup;
import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;

/**
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 *
 */
public class ExplorerPanel extends LayoutContainer {

	private Radio radioTree = null;
	private Radio radioSmartFolder = null;
	private AsyncTreePanel asycTreePanel = null;
	private SmartFolderPanel smartFolderPanel = null;
	private final RadioGroup radioGroup = new RadioGroup();
	private ContentPanel expPanel = null;
	private ToolBar toolBar = new ToolBar();
	private Radio radioMessages;
	private MessagesPanel messagesPanel;  
	private Button buttBulk = new Button(ConstantsExplorer.LOADER);

	
	public ExplorerPanel() {
		setBorders(false);
		initRadioButtons();
		initToolBar();
		this.asycTreePanel = new AsyncTreePanel();
		this.smartFolderPanel = new SmartFolderPanel();
		this.messagesPanel = new MessagesPanel();
		this.expPanel = new ContentPanel();
		this.expPanel.setHeaderVisible(false);
//		this.expPanel.setTitle("Explorer Panel");
//		this.expPanel.setLayout(new FitLayout());
		
		this.expPanel.setTopComponent(toolBar);
		
//		expPanel.add(radioGroup);
//		switchView(ViewSwitchType.Tree);
		
		smartFolderPanel.setVisible(false);
		messagesPanel.setVisible(false);
		asycTreePanel.setVisible(true);
		expPanel.add(asycTreePanel);
		expPanel.add(smartFolderPanel);
		expPanel.add(messagesPanel);
		
		//BULK
//		buttBulk.setIcon(Resources.getIconBulkUpdate());
		setLoadingBulk(false);
		addListnerOnBulk();
		
		add(expPanel);
	}
	
	
	/**
	 * If param is true instance only tree
	 * @param tree
	 */
	public ExplorerPanel(boolean tree) {

		setBorders(false);
		initRadioButtons();
		this.asycTreePanel = new AsyncTreePanel();
		this.asycTreePanel.loadRootItem(null); //load root item
		this.expPanel = new ContentPanel();
		this.expPanel.setHeaderVisible(false);
//		asycTreePanel.setVisible(true);
		expPanel.add(asycTreePanel);
		add(expPanel);
		
	}
	
	private void addListnerOnBulk() {
		buttBulk.addSelectionListener(new SelectionListener<ButtonEvent>() {
			
			@Override
			public void componentSelected(ButtonEvent ce) {
				BulkCreatorWindow.getInstance().show();
				
			}
		});
	}

	public void setLoadingBulk(boolean bool){
		
		if(!bool)
			buttBulk.setIcon(Resources.getIconLoadingOff());
		else
			buttBulk.setIcon(Resources.getIconLoading());
		
		toolBar.layout();
	}

	private void initRadioButtons() {

		this.radioTree = new Radio();
		this.radioSmartFolder = new Radio();
		this.radioMessages = new Radio();

//		radioTree.setId(ConstantsExplorer.TREE);
		radioTree.setBoxLabel(ConstantsExplorer.TREE);
		radioTree.setValue(true);
		radioTree.setValueAttribute(ConstantsExplorer.TREE);

//		radioSmartFolder.setId(ConstantsExplorer.SMARTFOLDER);
		radioSmartFolder.setBoxLabel(ConstantsExplorer.SMARTFOLDER);
		radioSmartFolder.setValueAttribute(ConstantsExplorer.SMARTFOLDER);
		
		radioMessages.setBoxLabel(ConstantsExplorer.MESSAGES);
		radioMessages.setValueAttribute(ConstantsExplorer.MESSAGES);

//		radioGroup.setFieldLabel("Afecto");
		radioGroup.add(radioTree);
		radioGroup.add(radioSmartFolder);
		radioGroup.add(radioMessages);
		radioGroup.setStyleAttribute("margin-left", "20px");

		radioGroup.addListener(Events.Change, new Listener<BaseEvent>() {
			public void handleEvent(BaseEvent be) {

				Radio selectedRadio = radioGroup.getValue();			
//				System.out.println("radio value:" +selectedRadio.getValueAttribute());
//				System.out.println("id" + selectedRadio.getId());
				if (selectedRadio.getValueAttribute().compareTo(ConstantsExplorer.SMARTFOLDER.toString()) == 0)
					switchView(ViewSwitchType.SmartFolder);
				else if(selectedRadio.getValueAttribute().compareTo(ConstantsExplorer.TREE.toString()) == 0)
						switchView(ViewSwitchType.Tree);
					else
						switchView(ViewSwitchType.Messages);
			}
		});

	}
	
	private void initToolBar(){
		
	    toolBar.add(radioGroup);
	    toolBar.add(buttBulk);
	}

	private void switchView(ViewSwitchType type) {

		if (type.compareTo(ViewSwitchType.Tree)==0) {
			smartFolderPanel.setVisible(false);
			messagesPanel.setVisible(false);
			asycTreePanel.setVisible(true);
			AppControllerExplorer.getEventBus().fireEvent(new SwitchViewEvent(ViewSwitchType.Tree));
//			asycTreePanel.setSearch(false);

		} else if (type.compareTo(ViewSwitchType.SmartFolder)==0){
			asycTreePanel.setVisible(false);
			messagesPanel.setVisible(false);
			smartFolderPanel.unPressedAllToogle();
			smartFolderPanel.setVisible(true);
			AppControllerExplorer.getEventBus().fireEvent(new SwitchViewEvent(ViewSwitchType.SmartFolder));

			// this.expPanel.setVisible(shortuctsPanel.getElement(), false);
			// this.expPanel.setVisible(asycTreePanel.getElement(), true);
		}
		else{	
			asycTreePanel.setVisible(false);
			smartFolderPanel.setVisible(false);
			messagesPanel.setVisible(true);
			AppControllerExplorer.getEventBus().fireEvent(new SwitchViewEvent(ViewSwitchType.Messages));
			
		}
			
	}

	public AsyncTreePanel getAsycTreePanel() {
		return asycTreePanel;
	}

	public SmartFolderPanel getShortuctsPanel() {
		return smartFolderPanel;
	}

	public SmartFolderPanel getSmartFolderPanel() {
		return smartFolderPanel;
	}

	public MessagesPanel getMessagesPanel() {
		return messagesPanel;
	}

	public void setMessagesPanel(MessagesPanel messagesPanel) {
		this.messagesPanel = messagesPanel;
	}
}
