package org.gcube.portlets.user.workspace.client.view.message;

import java.util.ArrayList;
import java.util.List;

import org.gcube.portlets.user.workspace.client.ConstantsExplorer;
import org.gcube.portlets.user.workspace.client.model.InfoContactModel;
import org.gcube.portlets.user.workspace.client.resources.Resources;

import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.Style.SelectionMode;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoader;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.ComponentEvent;
import com.extjs.gxt.ui.client.event.KeyListener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.CheckBoxSelectionModel;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.grid.filters.GridFilters;
import com.extjs.gxt.ui.client.widget.grid.filters.StringFilter;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;
import com.google.gwt.event.dom.client.KeyCodes;

/**
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 *
 */
public class LoadAndCheckContactsWindow extends Dialog {

	private CheckBoxSelectionModel<InfoContactModel> selectionModel = new CheckBoxSelectionModel<InfoContactModel>();
	private Grid<InfoContactModel> grid = null;
	private ContentPanel contentPanelGrid = new ContentPanel();
	private ListStore<InfoContactModel> store;
	private int widthDialog = 600;
	private int heightDialog = 500;
	private FormLayout formLayout = new FormLayout();
	private GridFilters filters = new GridFilters();
	private TextField<String> searchLogin = new TextField<String>();
	private StringFilter loginFilter = new StringFilter(ConstantsExplorer.LOGIN);
	private String emptyFilter = "Filter:";
	private Button btnActiveFilter = new Button(emptyFilter);

	public LoadAndCheckContactsWindow() {
	
	    setLayout(formLayout);
		setIcon(Resources.getIconCheckUser());
	    setButtonAlign(HorizontalAlignment.CENTER);
	    setHeading("Check Contacts");
	    setModal(true);
	    setBodyStyle("padding: 9px; background: none");
	    setWidth(widthDialog);
	    setHeight(heightDialog);
	    setResizable(false);
	    setButtons(Dialog.OKCANCEL);
	    
		addListners();
		store = new ListStore<InfoContactModel>();
		createGrid();
		createToolBar();
		add(contentPanelGrid);
	}

	private void createToolBar() {
		
		ToolBar toolBar = new ToolBar();
		
		searchLogin.setEmptyText("Search Login - Press key enter");
		searchLogin.setWidth(200);

		btnActiveFilter.setIcon(Resources.getIconRemoveFilter());
		btnActiveFilter.setToolTip("Remove current filter");
		setVisibleButtonFilter(false);
//		buttBiodiversity.setIconAlign(IconAlign.LEFT);

		toolBar.add(searchLogin);
		toolBar.add(btnActiveFilter);
	
		
		searchLogin.addKeyListener(new KeyListener(){
			@Override
			public void componentKeyDown(ComponentEvent event) {;
				if (event.getKeyCode()==KeyCodes.KEY_ENTER){
//					search(ConstantsExplorer.LOGIN, searchLogin.getValue());
					
					loginFilter.setValue(searchLogin.getValue());
					if(searchLogin.getValue()==null){
						btnActiveFilter.setText(emptyFilter);
						setVisibleButtonFilter(false);
					}
					else{
						btnActiveFilter.setText(emptyFilter + " "+ searchLogin.getValue());
						setVisibleButtonFilter(true);
					}
						
				}
			}
	
		});
		
		
		btnActiveFilter.addSelectionListener(new SelectionListener<ButtonEvent>() {

			@Override
			public void componentSelected(ButtonEvent ce) {
				
				loginFilter.setValue("");
				searchLogin.setValue("");
				btnActiveFilter.setText(emptyFilter);
				setVisibleButtonFilter(false);
				
			}
		
		});
		
		contentPanelGrid.setTopComponent(toolBar);
		
	}
	
	private void setVisibleButtonFilter(boolean bool){
		
		btnActiveFilter.setVisible(bool);
	}

	private void createGrid() {
		
		List<ColumnConfig> configs = new ArrayList<ColumnConfig>();

		configs.add(selectionModel.getColumn());

		ColumnConfig column = new ColumnConfig();
		column.setId(ConstantsExplorer.LOGIN);
		column.setHeader(ConstantsExplorer.GRIDCOLUMNLOGIN);
//		column.setWidth(150);
		configs.add(column);

		ColumnModel columnModel = new ColumnModel(configs);
		
		contentPanelGrid.setHeaderVisible(false);
		contentPanelGrid.setFrame(true);

		contentPanelGrid.setLayout(new FitLayout());
		contentPanelGrid.setHeight(418);
		contentPanelGrid.setScrollMode(Scroll.AUTO);
		
		// cp.setSize(600, 300);
		grid = new Grid<InfoContactModel>(store, columnModel);
		grid.setSelectionModel(selectionModel);
		grid.setBorders(true);
		grid.setColumnReordering(true);
		grid.getAriaSupport().setLabelledBy(contentPanelGrid.getHeader().getId() + "-label");
		grid.setHeight(418);
	
		//ADD FILTERING
		filters.setLocal(true);
		loginFilter = new StringFilter(ConstantsExplorer.LOGIN);
		filters.addFilter(loginFilter);
		grid.addPlugin(filters);
		
		grid.getView().setForceFit(true);
	
		grid.addPlugin(selectionModel);
		selectionModel.setSelectionMode(SelectionMode.MULTI);

		contentPanelGrid.add(grid);
		
		
	}

	public void createCheckBoxGrid(List<InfoContactModel> listAllContacts, List<InfoContactModel> listCheckedContacts)  {

		updateStore(listAllContacts);
		
		//Check existents list
		if(listCheckedContacts!=null){
			grid.getSelectionModel().select(listCheckedContacts, true);
		}
			
	}
	
	public void createCheckBoxGrid(List<InfoContactModel> listAllContacts)  {
		updateStore(listAllContacts);

	}

	private void updateStore(List<InfoContactModel> listAllContacts){
		
		store.removeAll();
		store.add(listAllContacts);	
	}


	public void addListners() {
			
        this.getButtonById(Dialog.CANCEL).addSelectionListener(new SelectionListener<ButtonEvent>() {

			@Override
			public void componentSelected(ButtonEvent ce) {
				hide();
			}
		});
        
	    
        this.getButtonById(Dialog.OK).addSelectionListener(new SelectionListener<ButtonEvent>() {

			@Override
			public void componentSelected(ButtonEvent ce) {

				hide();
			}
		});
	}

	public List<InfoContactModel> getCheckedList() {

		return selectionModel.getSelectedItems();
	}

//	public boolean isValidForm() {
//
//		if (selectionModel.getSelectedItems().size() > 0)
//			return true;
//
//		return false;
//
//	}
	
	public void search(String field, String text) {
		
		StringFilter filter = new StringFilter(field);
		filter.setValue(text);
		 
		// for prevent automatic reload while change filter
		filters.setAutoReload(false);
		filters.clearFilters();
		filters.addFilter(filter);
		filters.updateColumnHeadings();
		
//		System.out.println(" " + filters.getFilter("0"));
		 
		PagingLoader<PagingLoadResult<InfoContactModel>> loader =  (PagingLoader<PagingLoadResult<InfoContactModel>>) grid.getStore().getLoader();
		 
		loader.load();
		 
		// restore autoreload
		filters.setAutoReload(true);
	}
}
